﻿using EPiServer.Core;
using EPiServer.Logging;
using EPiServer.PlugIn;
using EPiServer.Scheduler;
using EPiServer.ServiceLocation;
using FotoWare.PlugIns.Episerver.Interfaces;
using System;
using System.Linq;
using System.Threading;

namespace FotoWare.PlugIns.Episerver.ArchiveImport
{
    [ScheduledPlugIn(DisplayName = "FotoWare - Import assets", Description = "Imports all assets from FotoWare from specified archives")]
    public class FotoWareImportArchiveScheduledJob : ScheduledJobBase
    {
        private static readonly ILogger Logger = LogManager.GetLogger();
        private readonly IFotoWareSettings _fotowareSettings;
        private readonly IFotoWareSynchronizer _fotowareSynchronizer;
        private bool _stopSignaled;
        protected bool ForceUpdate = false;

        public FotoWareImportArchiveScheduledJob()
        : this(ServiceLocator.Current.GetInstance<IFotoWareSettings>(), ServiceLocator.Current.GetInstance<IFotoWareSynchronizer>())
        { }

        public FotoWareImportArchiveScheduledJob(IFotoWareSettings fotowareSettings, IFotoWareSynchronizer fotowareSynchronizeManager)
        {
            IsStoppable = true;
            _fotowareSettings = fotowareSettings;
            _fotowareSynchronizer = fotowareSynchronizeManager;
        }

        /// <summary>
        /// Called when a user clicks on Stop for a manually started job, or when ASP.NET shuts down.
        /// </summary>
        public override void Stop()
        {
            _stopSignaled = true;
        }

        /// <summary>
        /// Called when a scheduled job executes
        /// </summary>
        /// <returns>A status message to be stored in the database log and visible from admin mode</returns>
        public override string Execute()
        {
            ValidateJobSettings();
            //Call OnStatusChanged to periodically notify progress of job for manually started jobs
            OnStatusChanged($"Starting execution of {this.GetType()}");
            var archiveCallCount = 0;
            var checkedItemsCount = 0;
            var changedItemsCount = 0;
            var failedItemsCount = 0; 

            foreach (string archiveUrl in _fotowareSettings.ArchiveUrlsToSynchronize)
            {
                string url = archiveUrl;

                while (!string.IsNullOrEmpty(url))
                {
                    //For long running jobs periodically check if stop is signaled and if so stop execution
                    if (_stopSignaled)
                    {
                        return "Stop of job was called";
                    }

                    int retryAttempts = 3;
                    SynchronizationBatchReturnValue returnValue = null;

                    while (retryAttempts > 0)
                    {
                        returnValue = _fotowareSynchronizer.SynchronizeItems(url, archiveCallCount <= 30, ForceUpdate).Result;
                        if (returnValue.FailedCallingFotoWare)
                        {
                            if(retryAttempts == 1)
                            { 
                                return "Failed calling FotoWare during job, job abandoned. Called FotoWare " + archiveCallCount + " times, found a total of " + checkedItemsCount + " assets, updated " + changedItemsCount + " of these with " + failedItemsCount + " failed assets."; ;
                            }

                            retryAttempts--;
                            Logger.Warning("FotoWare integration: Failed calling FotoWare, trying again. Url is: " + url);
                            Thread.Sleep(2000);
                        }
                        else
                        {
                            break;
                        }
                    }

                    url = returnValue?.NextPageUrl;
                    archiveCallCount++;
                    checkedItemsCount += returnValue.CheckedItemCount;
                    changedItemsCount += returnValue.ChangedItemCount;
                    failedItemsCount += returnValue.FailedItemCount;
                    OnStatusChanged(
                        $"Job in progress: Called FotoWare {archiveCallCount} times, found a total of {checkedItemsCount} assets from FotoWare, updated {changedItemsCount} of these with {failedItemsCount} failed assets.");
                }
              
                Logger.Debug("FotoWare: Done synchronizing archive with url: " + archiveUrl);
            }

            if (_fotowareSettings.ArchiveUrlsToSynchronize.Count() == 0)
            {
                return "FotoWare:ArchiveUrlsToSynchronize not set in appsettings, nothing was synchronized";
            }


            return "Called FotoWare " + archiveCallCount + " times, found a total of " + checkedItemsCount + " assets from FotoWare, updated " + changedItemsCount + " of these with " + failedItemsCount + " failed assets.";
        }

        private void ValidateJobSettings()
        {
            if (string.IsNullOrEmpty(_fotowareSettings.ApiToken))
            {
                throw new ArgumentException("ApiToken is not set. Please give it a value in AppSettings.");
            }
            if (ContentReference.IsNullOrEmpty(_fotowareSettings.RootFolderId))
            {
                throw new ArgumentException("RootFolderId is not set. Please give it a value in AppSettings.");
            }
            if (string.IsNullOrWhiteSpace(_fotowareSettings.FotoWareTenantUrl))
            {
                throw new ArgumentException("TenantUrl is not set. Please give it a value in AppSettings.");
            }

            var type = typeof(IFotoWareMedia);
            var imageType = AppDomain.CurrentDomain.GetAssemblies()
                .SelectMany(s => s.GetTypes())
                .FirstOrDefault(x =>
                    type.IsAssignableFrom(x) &
                    !x.IsInterface &
                    !x.Name.EndsWith("Proxy"));

            if (imageType == null)
            {
                throw new ArgumentException("A content type which implements the interface " + typeof(IFotoWareMedia).Name + " could not be found.");
            }

        }
    }
}